// Catapult game

#include <Servo.h>
#include <LiquidCrystal.h>

const int anglePotPin = A0;             // Pin connected to angle potentiometer
const int powerPotPin = A1;             // Pin connected to power potentiometer
const int angleServoPin = 3;            // Pin connected to angle servo
const int powerServoPin = 4;            // Pin connected to power servo
const int buttonPin = 2;                // Pin connected to button
const int pressureSensorPin = A2;       // Pin connected to pressure sensor

const int maxPower = 90;                // Maximum power of the catapult
int pressureThreshold = 10;             // Pressure sensor threshold
const int launchDelay = 2000;           // Delay before resetting the servo (in milliseconds)
int Reading;                            // Setting up Reading variable

Servo catapultServo;                    // Setting up servos
Servo angleServo;

LiquidCrystal lcd(12, 11, 10, 9, 8, 7); // LCD pins

unsigned long startTime = 0;            // Time when the game starts
bool launchPending = false;             // Flag to indicate if a launch is pending

void setup() {
  pinMode(anglePotPin, INPUT);          // Setting up pinmodes
  pinMode(powerPotPin, INPUT);
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(pressureSensorPin, INPUT);

  Serial.begin(9600);
  
  angleServo.attach(angleServoPin);     // Attaching servos
  catapultServo.attach(powerServoPin);
  
  lcd.begin(16, 2);                     // Initializing LCD
  
  // Display starting message
  lcd.print("Catapult Game");
  lcd.setCursor(0, 1);
  lcd.print("Press button...");
}

void loop() {
  unsigned long InitialTime = millis(); // Grabbing time before button is pressed.
  // Check if button is pressed to start the game
  if (digitalRead(buttonPin) == HIGH) {
    lcd.clear();
    while (pressureThreshold > Reading) {

     // Reads pressure sensor pin and prints to serial
     int Reading = analogRead(pressureSensorPin);
      Serial.println(Reading);

     // Ends the game once the pressure reading is greater than the threshold
     if (pressureThreshold < Reading) {
       Serial.print("Reading higher than Threshold!");
       break;
      }

     // Read potentiometer values
     int angle = map(analogRead(anglePotPin), 0, 1023, 0, 180);
    
     // Apply angle to servo motor
     angleServo.write(angle);

     // Check if button is pressed, begin launch sequence
     if (digitalRead(buttonPin) == HIGH) {
       launchPending = true;
       int power = map(analogRead(powerPotPin), 0, 1023, 0, 60);
       catapultServo.write(power);
       delay(launchDelay); // Wait for the launch delay
       catapultServo.write(0); // Reset servo back to 0
       launchPending = false;
      }
     
     // Subtracts initiation time from total time, producing total in-game time
     unsigned long duration = millis() - InitialTime;
      
     // Calculate minutes and seconds
     int minutes = (duration) / 60000;
     int seconds = (duration % 60000) / 1000;

     // Display current game time and game started screen
     lcd.setCursor(0,0);
     lcd.print("Game started!");
     lcd.setCursor(0, 1);
     lcd.print("Time: ");
     if (minutes < 10) {
       lcd.print("0");}
     lcd.print(minutes);
     lcd.print(":");
     if (seconds < 10) {
       lcd.print("0"); }
     lcd.print(seconds);
    }

   // Display game over screen and final time
   unsigned long duration = millis() - InitialTime;
   int minutes = (duration) / 60000;
   int seconds = (duration % 60000) / 1000;

   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("Game over!");
   lcd.setCursor(0,1);
   lcd.print("Time: ");
   if (minutes < 10) {
     lcd.print("0");}
   lcd.print(minutes);
   lcd.print(":");
   if (seconds < 10) {
     lcd.print("0"); }
   lcd.print(seconds);
  }
}